@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2023 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -43,12 +43,6 @@
 #endif
 #include "zlib.h"
 
-#ifdef WITH_LIBUNWIND
-#define UNW_LOCAL_ONLY 1
-#include <libunwind.h>
-#define UNWIND_BACKTRACE_DEPTH 256
-#endif
-
 #ifdef HAVE_UCONTEXT_H
 #include <ucontext.h>
 #elif defined(HAVE_SYS_UCONTEXT_H)
@@ -1610,56 +1604,6 @@ void rspamd_worker_init_monitored(struct rspamd_worker *worker,
 
 #ifdef HAVE_SA_SIGINFO
 
-#ifdef WITH_LIBUNWIND
-static void
-rspamd_print_crash(ucontext_t *uap)
-{
-	unw_cursor_t cursor;
-	unw_word_t ip, off;
-	guint level;
-	gint ret;
-
-	if ((ret = unw_init_local(&cursor, uap)) != 0) {
-		msg_err("unw_init_local: %d", ret);
-
-		return;
-	}
-
-	level = 0;
-	ret = 0;
-
-	for (;;) {
-		char name[128];
-
-		if (level >= UNWIND_BACKTRACE_DEPTH) {
-			break;
-		}
-
-		unw_get_reg(&cursor, UNW_REG_IP, &ip);
-		ret = unw_get_proc_name(&cursor, name, sizeof(name), &off);
-
-		if (ret == 0) {
-			msg_err("%d: %p: %s()+0x%xl",
-					level, ip, name, (uintptr_t) off);
-		}
-		else {
-			msg_err("%d: %p: <unknown>", level, ip);
-		}
-
-		level++;
-		ret = unw_step(&cursor);
-
-		if (ret <= 0) {
-			break;
-		}
-	}
-
-	if (ret < 0) {
-		msg_err("unw_step_ptr: %d", ret);
-	}
-}
-#endif
-
 static struct rspamd_main *saved_main = NULL;
 static gboolean
 rspamd_crash_propagate(gpointer key, gpointer value, gpointer unused)
@@ -1672,6 +1616,11 @@ rspamd_crash_propagate(gpointer key, gpointer value, gpointer unused)
 	return TRUE;
 }
 
+#ifdef BACKWARD_ENABLE
+/* See backtrace.cxx */
+extern void rspamd_print_crash(void);
+#endif
+
 static void
 rspamd_crash_sig_handler(int sig, siginfo_t *info, void *ctx)
 {
@@ -1684,8 +1633,8 @@ rspamd_crash_sig_handler(int sig, siginfo_t *info, void *ctx)
 			"pid: %P, trace: ",
 			sig, strsignal(sig), pid);
 	(void) uap;
-#ifdef WITH_LIBUNWIND
-	rspamd_print_crash(uap);
+#ifdef BACKWARD_ENABLE
+	rspamd_print_crash();
 #endif
 	msg_err("please see Rspamd FAQ to learn how to dump core files and how to "
 			"fill a bug report");
