@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2023 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -35,6 +35,7 @@ static const gchar *M = "redis learn cache";
 struct rspamd_redis_cache_ctx {
 	lua_State *L;
 	struct rspamd_statfile_config *stcf;
+	const gchar *username;
 	const gchar *password;
 	const gchar *dbname;
 	const gchar *redis_object;
@@ -77,7 +78,15 @@ static void
 rspamd_redis_cache_maybe_auth(struct rspamd_redis_cache_ctx *ctx,
 							  redisAsyncContext *redis)
 {
-	if (ctx->password) {
+	if (ctx->username) {
+		if (ctx->password) {
+			redisAsyncCommand(redis, NULL, NULL, "AUTH %s %s", ctx->username, ctx->password);
+		}
+		else {
+			msg_warn("Redis requires a password when username is supplied");
+		}
+	}
+	else if (ctx->password) {
 		redisAsyncCommand(redis, NULL, NULL, "AUTH %s", ctx->password);
 	}
 	if (ctx->dbname) {
@@ -275,7 +284,7 @@ rspamd_stat_cache_redis_init(struct rspamd_stat_ctx *ctx,
 
 	/* Now try global redis settings */
 	if (!ret) {
-		obj = ucl_object_lookup(cfg->rcl_obj, "redis");
+		obj = ucl_object_lookup(cfg->cfg_ucl_obj, "redis");
 
 		if (obj) {
 			const ucl_object_t *specific_obj;
@@ -328,6 +337,14 @@ rspamd_stat_cache_redis_init(struct rspamd_stat_ctx *ctx,
 	}
 	lua_pop(L, 1);
 
+	lua_pushstring(L, "username");
+	lua_gettable(L, -2);
+	if (lua_type(L, -1) == LUA_TSTRING) {
+		cache_ctx->username = rspamd_mempool_strdup(cfg->cfg_pool,
+													lua_tostring(L, -1));
+	}
+	lua_pop(L, 1);
+
 	lua_pushstring(L, "password");
 	lua_gettable(L, -2);
 	if (lua_type(L, -1) == LUA_TSTRING) {
