@@ -1,11 +1,11 @@
-/*-
- * Copyright 2017 Vsevolod Stakhov
+/*
+ * Copyright 2023 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -19,6 +19,7 @@
 
 #include "ottery.h"
 #include "cryptobox.h"
+#include "contrib/libev/ev.h"
 
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
@@ -2231,6 +2232,7 @@ rspamd_fstring_gzip(rspamd_fstring_t **in)
 	strm.avail_out = sizeof(temp) > buf->allocated ? buf->allocated : sizeof(temp);
 	ret = deflate(&strm, Z_FINISH);
 	if (ret == Z_STREAM_ERROR) {
+		deflateEnd(&strm);
 		return FALSE;
 	}
 
@@ -2247,6 +2249,8 @@ rspamd_fstring_gzip(rspamd_fstring_t **in)
 		if (ret != Z_BUF_ERROR || strm.avail_in == 0) {
 			buf->len = strm.next_out - (unsigned char *) buf->str;
 			*in = buf;
+			deflateEnd(&strm);
+
 			return ret == Z_STREAM_END;
 		}
 	}
@@ -2267,6 +2271,7 @@ rspamd_fstring_gzip(rspamd_fstring_t **in)
 	g_free(hold);
 	buf->len = strm.next_out - (unsigned char *) buf->str;
 	*in = buf;
+	deflateEnd(&strm);
 
 	return ret == Z_STREAM_END;
 }
@@ -2291,21 +2296,20 @@ rspamd_fstring_gunzip(rspamd_fstring_t **in)
 	gsize total_out = 0;
 
 	do {
-		strm.next_out = out->str;
-		strm.avail_out = out->allocated;
+		strm.next_out = out->str + total_out;
+		strm.avail_out = out->allocated - total_out;
 
 		ret = inflate(&strm, Z_NO_FLUSH);
 		if (ret != Z_OK && ret != Z_STREAM_END && ret != Z_BUF_ERROR) {
 			break;
 		}
 
-		gsize out_size = out->allocated - strm.avail_out;
-		if (total_out + out_size > out->allocated) {
-			out = rspamd_fstring_grow(out, total_out + out_size);
+		gsize out_remain = strm.avail_out;
+		total_out = out->allocated - out_remain;
+		if (out_remain == 0 && ret != Z_STREAM_END) {
+			out = rspamd_fstring_grow(out, out->allocated * 2);
 		}
 
-		total_out += out_size;
-
 	} while (ret != Z_STREAM_END);
 
 	if (ret == Z_STREAM_END) {
