@@ -83,16 +83,20 @@ end
 local function validate_dns(lstr)
   if lstr:match('%.%.') then
     -- two dots in a row
-    return false
+    return false, "two dots in a row"
   end
   if not rspamd_util.is_valid_utf8(lstr) then
     -- invalid utf8 detected
-    return false
+    return false, "invalid utf8"
   end
   for v in lstr:gmatch('[^%.]+') do
-    if v:len() > 63 or v:match('^-') or v:match('-$') then
-      -- too long label or weird labels
-      return false
+    if v:len() > 63 then
+      -- too long label
+      return false, "too long label"
+    end
+    if v:match('^-') or v:match('-$') then
+      -- dash at the beginning or end of label
+      return false, "dash at the beginning or end of label"
     end
   end
   return true
@@ -142,7 +146,9 @@ local function gen_check_rcvd_conditions(rbl, received_total)
   local nmatch_flags = rbl.received_nflags
 
   local function basic_received_check(rh)
-    if not (rh.real_ip and rh.real_ip:is_valid()) then return false end
+    if not (rh.real_ip and rh.real_ip:is_valid()) then
+      return false
+    end
     if ((rh.real_ip:get_version() == 6 and rbl.ipv6) or
         (rh.real_ip:get_version() == 4 and rbl.ipv4)) and
         ((rbl.exclude_private_ips and not rh.real_ip:is_local()) or
@@ -155,7 +161,9 @@ local function gen_check_rcvd_conditions(rbl, received_total)
   end
 
   local function positioned_received_check(rh, pos)
-    if not rh or not basic_received_check(rh) then return false end
+    if not rh or not basic_received_check(rh) then
+      return false
+    end
     local got_flags = rh.flags or E
     if min_pos then
       if min_pos < 0 then
@@ -200,7 +208,6 @@ local function gen_check_rcvd_conditions(rbl, received_total)
     return true
   end
 
-
   if not (max_pos or min_pos or match_flags or nmatch_flags) then
     return basic_received_check
   else
@@ -208,7 +215,43 @@ local function gen_check_rcvd_conditions(rbl, received_total)
   end
 end
 
-local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_table_elt)
+local matchers = {}
+
+matchers.radix = function(_, _, real_ip, map)
+  return map and map:get_key(real_ip) or false
+end
+
+matchers.equality = function(codes, to_match)
+  if type(codes) ~= 'table' then return codes == to_match end
+  for _, ip in ipairs(codes) do
+    if to_match == ip then
+      return true
+    end
+  end
+  return false
+end
+
+matchers.luapattern = function(codes, to_match)
+  if type(codes) ~= 'table' then
+    return string.find(to_match, '^' .. codes .. '$') and true or false
+  end
+  for _, pattern in ipairs(codes) do
+    if string.find(to_match, '^' .. pattern .. '$') then
+      return true
+    end
+  end
+  return false
+end
+
+matchers.regexp = function(_, to_match, _, map)
+  return map and map:get_key(to_match) or false
+end
+
+matchers.glob = function(_, to_match, _, map)
+  return map and map:get_key(to_match) or false
+end
+
+local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_table_elt, match)
   local function make_option(ip, label)
     if ip then
       return string.format('%s:%s:%s',
@@ -267,15 +310,17 @@ local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_tabl
     return
   end
 
-  for _,result in ipairs(results) do
+  local returncodes_maps = rbl.returncodes_maps or {}
+
+  for _, result in ipairs(results) do
     local ipstr = result:to_string()
     lua_util.debugm(N, task, '%s DNS result %s', to_resolve, ipstr)
     local foundrc = false
     -- Check return codes
     if rbl.returnbits then
       local ipnum = result:to_number()
-      for s,bits in pairs(rbl.returnbits) do
-        for _,check_bit in ipairs(bits) do
+      for s, bits in pairs(rbl.returnbits) do
+        for _, check_bit in ipairs(bits) do
           if bit.band(ipnum, check_bit) == check_bit then
             foundrc = true
             insert_results(s)
@@ -285,12 +330,10 @@ local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_tabl
       end
     elseif rbl.returncodes then
       for s, codes in pairs(rbl.returncodes) do
-        for _,v in ipairs(codes) do
-          if string.find(ipstr, '^' .. v .. '$') then
-            foundrc = true
-            insert_results(s)
-            break
-          end
+        local res = match(codes, ipstr, result, returncodes_maps[s])
+        if res then
+          foundrc = true
+          insert_results(s)
         end
       end
     end
@@ -370,7 +413,7 @@ local function gen_rbl_callback(rule)
         nreq.what[label] = true
       end
 
-      return true,nreq -- Duplicate
+      return true, nreq -- Duplicate
     else
       local nreq
 
@@ -384,7 +427,7 @@ local function gen_rbl_callback(rule)
             n = processed,
             orig = req_str,
             resolve_ip = resolve_ip,
-            what = {[label] = true},
+            what = { [label] = true },
           }
           requests_table[req] = nreq
         end
@@ -407,7 +450,7 @@ local function gen_rbl_callback(rule)
           n = to_resolve,
           orig = req_str,
           resolve_ip = resolve_ip,
-          what = {[label] = true},
+          what = { [label] = true },
         }
         requests_table[req] = nreq
       end
@@ -431,7 +474,9 @@ local function gen_rbl_callback(rule)
 
   local function check_required_symbols(task, _)
     if rule.require_symbols then
-      return fun.all(function(sym) task:has_symbol(sym) end, rule.require_symbols)
+      return fun.all(function(sym)
+        task:has_symbol(sym)
+      end, rule.require_symbols)
     end
 
     return true
@@ -463,7 +508,8 @@ local function gen_rbl_callback(rule)
     local helo = task:get_helo()
 
     if not helo then
-      return false
+      -- Avoid pipeline breaking
+      return true
     end
 
     add_dns_request(task, helo, true, false, requests_table,
@@ -483,7 +529,7 @@ local function gen_rbl_callback(rule)
         mime_from_domain = ((task:get_from('mime') or E)[1] or E).domain
         if mime_from_domain then
           local mime_from_domain_tld = rule.url_full_hostname and
-               mime_from_domain or rspamd_util.get_tld(mime_from_domain)
+              mime_from_domain or rspamd_util.get_tld(mime_from_domain)
 
           if rule.url_compose_map then
             mime_from_domain = rule.url_compose_map:process_url(task, mime_from_domain_tld, mime_from_domain)
@@ -495,7 +541,7 @@ local function gen_rbl_callback(rule)
 
       for _, d in ipairs(das[1].options) do
 
-        local domain,result = d:match('^([^%:]*):([%+%-%~])$')
+        local domain, result = d:match('^([^%:]*):([%+%-%~])$')
 
         -- We must ignore bad signatures, omg
         if domain and result and result == '+' then
@@ -515,7 +561,7 @@ local function gen_rbl_callback(rule)
 
             if mime_from_domain and mime_from_domain == domain_tld then
               add_dns_request(task, domain_tld, true, false, requests_table,
-              'dkim', whitelist)
+                  'dkim', whitelist)
             end
           else
             if rule.dkim_domainonly then
@@ -557,7 +603,31 @@ local function gen_rbl_callback(rule)
       no_cache = true,
     }
 
-    if not rule.urls then
+    if rule.numeric_urls then
+      if rule.content_urls then
+        if not rule.images then
+          ex_params.flags_mode = 'explicit'
+          ex_params.flags = { 'numeric' }
+          ex_params.filter = function(url)
+            return (bit.band(url:get_flags_num(), url_flag_bits.image) == 0)
+          end
+        else
+          ex_params.filter = function(url)
+            return (bit.band(url:get_flags_num(), url_flag_bits.numeric) ~= 0)
+          end
+        end
+      elseif rule.images then
+        ex_params.filter = function(url)
+          return (bit.band(url:get_flags_num(), url_flag_bits.numeric) ~= 0)
+        end
+      else
+        ex_params.flags_mode = 'explicit'
+        ex_params.flags = { 'numeric' }
+        ex_params.filter = function(url)
+          return (bit.band(url:get_flags_num(), url_flag_bits.content) == 0)
+        end
+      end
+    elseif not rule.urls and (rule.content_urls or rule.images) then
       ex_params.flags_mode = 'explicit'
       ex_params.flags = {}
       if rule.content_urls then
@@ -570,7 +640,7 @@ local function gen_rbl_callback(rule)
 
     local urls = lua_util.extract_specific_urls(ex_params)
 
-    for _,u in ipairs(urls) do
+    for _, u in ipairs(urls) do
       local flags = u:get_flags_num()
 
       if bit.band(flags, url_flag_bits.numeric) ~= 0 then
@@ -615,14 +685,14 @@ local function gen_rbl_callback(rule)
   end
 
   local function check_received(task, requests_table, whitelist)
-    local received = fun.filter(function(h)
+    local received = fun             .filter(function(h)
       return not h['flags']['artificial']
     end, task:get_received_headers()):totable()
 
     local received_total = #received
     local check_conditions = gen_check_rcvd_conditions(rule, received_total)
 
-    for pos,rh in ipairs(received) do
+    for pos, rh in ipairs(received) do
       if check_conditions(rh, pos) then
         add_dns_request(task, rh.real_ip, false, true,
             requests_table, 'received',
@@ -636,7 +706,7 @@ local function gen_rbl_callback(rule)
   local function check_rdns(task, requests_table, whitelist)
     local hostname = task:get_hostname()
     if hostname == nil or hostname == 'unknown' then
-      return false
+      return true
     end
 
     add_dns_request(task, hostname, true, false,
@@ -650,13 +720,13 @@ local function gen_rbl_callback(rule)
       local res = selector(task)
 
       if res and type(res) == 'table' then
-          for _,r in ipairs(res) do
-            add_dns_request(task, r, false, false, requests_table,
-                    selector_label, whitelist)
-          end
+        for _, r in ipairs(res) do
+          add_dns_request(task, r, false, false, requests_table,
+              selector_label, whitelist)
+        end
       elseif res then
         add_dns_request(task, res, false, false,
-                requests_table, selector_label, whitelist)
+            requests_table, selector_label, whitelist)
       end
     end
 
@@ -704,7 +774,9 @@ local function gen_rbl_callback(rule)
     local ex_params = {
       task = task,
       limit = rule.requests_limit,
-      filter = function(u) return u:get_protocol() == 'mailto' end,
+      filter = function(u)
+        return u:get_protocol() == 'mailto'
+      end,
       need_emails = true,
       prefix = 'rbl_email'
     }
@@ -718,7 +790,7 @@ local function gen_rbl_callback(rule)
 
     local emails = lua_util.extract_specific_urls(ex_params)
 
-    for _,email in ipairs(emails) do
+    for _, email in ipairs(emails) do
       local domain
       if rule.emails_domainonly and not rule.url_full_hostname then
         if rule.url_compose_map then
@@ -797,7 +869,7 @@ local function gen_rbl_callback(rule)
     description[#description + 1] = 'replyto'
   end
 
-  if rule.urls or rule.content_urls or rule.images then
+  if rule.urls or rule.content_urls or rule.images or rule.numeric_urls then
     pipeline[#pipeline + 1] = check_urls
     description[#description + 1] = 'urls'
   end
@@ -822,6 +894,11 @@ local function gen_rbl_callback(rule)
     description[#description + 1] = 'selector'
   end
 
+  if not rule.returncodes_matcher then
+    rule.returncodes_matcher = 'equality'
+  end
+  local match = matchers[rule.returncodes_matcher]
+
   local callback_f = function(task)
     -- DNS requests to issue (might be hashed afterwards)
     local dns_req = {}
@@ -829,12 +906,12 @@ local function gen_rbl_callback(rule)
 
     local function gen_rbl_dns_callback(resolve_table_elt)
       return function(_, to_resolve, results, err)
-        rbl_dns_process(task, rule, to_resolve, results, err, resolve_table_elt)
+        rbl_dns_process(task, rule, to_resolve, results, err, resolve_table_elt, match)
       end
     end
 
     -- Execute functions pipeline
-    for i,f in ipairs(pipeline) do
+    for i, f in ipairs(pipeline) do
       if not f(task, dns_req, whitelist) then
         lua_util.debugm(N, task,
             "skip rbl check: %s; pipeline condition %s returned false",
@@ -853,12 +930,12 @@ local function gen_rbl_callback(rule)
     local function gen_rbl_ip_dns_callback(orig_resolve_table_elt)
       return function(_, _, results, err)
         if not err then
-          for _,dns_res in ipairs(results) do
+          for _, dns_res in ipairs(results) do
             -- Check if we have rspamd{ip} userdata
             if type(dns_res) == 'userdata' then
               -- Add result as an actual RBL request
               local label = next(orig_resolve_table_elt.what)
-              local dup,nreq = add_dns_request(task, dns_res, false, true,
+              local dup, nreq = add_dns_request(task, dns_res, false, true,
                   resolved_req, label)
               -- Add original name
               if not dup then
@@ -873,7 +950,8 @@ local function gen_rbl_callback(rule)
         if nresolved == 0 then
           -- Emit real RBL requests as there are no ip resolution requests
           for name, req in pairs(resolved_req) do
-            if validate_dns(req.n) then
+            local val_res, val_error = validate_dns(req.n)
+            if val_res then
               lua_util.debugm(N, task, "rbl %s; resolve %s -> %s",
                   rule.symbol, name, req.n)
               r:resolve_a({
@@ -883,8 +961,8 @@ local function gen_rbl_callback(rule)
                 forced = req.forced
               })
             else
-              rspamd_logger.warnx(task, 'cannot send invalid DNS request %s for %s',
-                  req.n, rule.symbol)
+              rspamd_logger.warnx(task, 'cannot send invalid DNS request %s for %s: %s',
+                  req.n, rule.symbol, val_error)
             end
           end
         end
@@ -892,7 +970,8 @@ local function gen_rbl_callback(rule)
     end
 
     for name, req in pairs(dns_req) do
-      if validate_dns(req.n) then
+      local val_res, val_error = validate_dns(req.n)
+      if val_res then
         lua_util.debugm(N, task, "rbl %s; resolve %s -> %s",
             rule.symbol, name, req.n)
 
@@ -925,21 +1004,27 @@ local function gen_rbl_callback(rule)
         end
 
       else
-        rspamd_logger.warnx(task, 'cannot send invalid DNS request %s for %s',
-            req.n, rule.symbol)
+        rspamd_logger.warnx(task, 'cannot send invalid DNS request %s for %s: %s',
+            req.n, rule.symbol, val_error)
       end
     end
   end
 
-  return callback_f,string.format('checks: %s', table.concat(description, ','))
+  return callback_f, string.format('checks: %s', table.concat(description, ','))
 end
 
+local map_match_types = {
+  glob = true,
+  radix = true,
+  regexp = true,
+}
+
 local function add_rbl(key, rbl, global_opts)
   if not rbl.symbol then
     rbl.symbol = key:upper()
   end
 
-  local flags_tbl = {'no_squeeze'}
+  local flags_tbl = { 'no_squeeze' }
   if rbl.is_whitelist then
     flags_tbl[#flags_tbl + 1] = 'nice'
   end
@@ -954,7 +1039,7 @@ local function add_rbl(key, rbl, global_opts)
 
     rbl.selectors = {}
     if type(rbl.selector) ~= 'table' then
-      rbl.selector = {['selector'] = rbl.selector}
+      rbl.selector = { ['selector'] = rbl.selector }
     end
 
     for selector_label, selector in pairs(rbl.selector) do
@@ -969,7 +1054,7 @@ local function add_rbl(key, rbl, global_opts)
           rbl.selector_flatten = true
         end
         local sel = selectors.create_selector_closure(rspamd_config, selector, '',
-                rbl.selector_flatten)
+            rbl.selector_flatten)
 
         if not sel then
           rspamd_logger.errx('invalid selector for rbl rule %s: %s', key, selector)
@@ -1011,6 +1096,19 @@ local function add_rbl(key, rbl, global_opts)
         def_type, rbl.symbol)
   end
 
+  local match_type = rbl.returncodes_matcher
+  if match_type and rbl.returncodes and map_match_types[match_type] then
+    if not rbl.returncodes_maps then
+      rbl.returncodes_maps = {}
+    end
+    for label, v in pairs(rbl.returncodes) do
+      if type(v) ~= 'table' then
+        v = {v}
+      end
+      rbl.returncodes_maps[label] = lua_maps.map_add_from_ucl(v, match_type, string.format('%s_%s RBL returncodes', label, rbl.symbol))
+    end
+  end
+
   if rbl.url_compose_map then
     local lua_urls_compose = require "lua_urls_compose"
     rbl.url_compose_map = lua_urls_compose.add_composition_map(rspamd_config, rbl.url_compose_map)
@@ -1031,22 +1129,23 @@ local function add_rbl(key, rbl, global_opts)
         rbl.symbol)
   end
 
-  local callback,description = gen_rbl_callback(rbl)
+  local callback, description = gen_rbl_callback(rbl)
 
   if callback then
     local id
 
     if rbl.symbols_prefixes then
-      id = rspamd_config:register_symbol{
+      id = rspamd_config:register_symbol {
         type = 'callback',
         callback = callback,
+        groups = { 'rbl' },
         name = rbl.symbol .. '_CHECK',
         flags = table.concat(flags_tbl, ',')
       }
 
-      for _,prefix in pairs(rbl.symbols_prefixes) do
+      for _, prefix in pairs(rbl.symbols_prefixes) do
         -- For unknown results...
-        rspamd_config:register_symbol{
+        rspamd_config:register_symbol {
           type = 'virtual',
           parent = id,
           group = 'rbl',
@@ -1062,10 +1161,11 @@ local function add_rbl(key, rbl, global_opts)
             rbl.symbol, rbl.is_whitelist, rbl.ignore_whitelist)
       end
     else
-      id = rspamd_config:register_symbol{
+      id = rspamd_config:register_symbol {
         type = 'callback',
         callback = callback,
         name = rbl.symbol,
+        groups = { 'rbl' },
         group = 'rbl',
         score = 0,
         flags = table.concat(flags_tbl, ',')
@@ -1079,24 +1179,25 @@ local function add_rbl(key, rbl, global_opts)
       end
     end
 
-
     rspamd_logger.infox(rspamd_config, 'added rbl rule %s: %s',
         rbl.symbol, description)
     lua_util.debugm(N, rspamd_config, 'rule dump for %s: %s',
         rbl.symbol, rbl)
 
+    local check_sym = rbl.symbols_prefixes and rbl.symbol .. '_CHECK' or rbl.symbol
+
     if rbl.dkim then
-      rspamd_config:register_dependency(rbl.symbol, 'DKIM_CHECK')
+      rspamd_config:register_dependency(check_sym, 'DKIM_CHECK')
     end
 
     if rbl.require_symbols then
-      for _,dep in ipairs(rbl.require_symbols) do
-        rspamd_config:register_dependency(rbl.symbol, dep)
+      for _, dep in ipairs(rbl.require_symbols) do
+        rspamd_config:register_dependency(check_sym, dep)
       end
     end
 
     -- Failure symbol
-    rspamd_config:register_symbol{
+    rspamd_config:register_symbol {
       type = 'virtual',
       flags = 'nostat',
       name = rbl.symbol .. '_FAIL',
@@ -1109,7 +1210,7 @@ local function add_rbl(key, rbl, global_opts)
         if s ~= rbl.symbol then
           -- hack
 
-          rspamd_config:register_symbol{
+          rspamd_config:register_symbol {
             type = 'virtual',
             parent = id,
             name = s,
@@ -1140,7 +1241,7 @@ local function add_rbl(key, rbl, global_opts)
       end
 
       if rbl.symbols_prefixes then
-        for _,prefix in pairs(rbl.symbols_prefixes) do
+        for _, prefix in pairs(rbl.symbols_prefixes) do
           process_specific_suffix(prefix .. '_' .. suffix)
         end
       else
@@ -1150,13 +1251,13 @@ local function add_rbl(key, rbl, global_opts)
     end
 
     if rbl.returncodes then
-      for s,_ in pairs(rbl.returncodes) do
+      for s, _ in pairs(rbl.returncodes) do
         process_return_code(s)
       end
     end
 
     if rbl.returnbits then
-      for s,_ in pairs(rbl.returnbits) do
+      for s, _ in pairs(rbl.returnbits) do
         process_return_code(s)
       end
     end
@@ -1195,12 +1296,25 @@ if opts.rules and opts.rbls then
   opts.rbls = lua_util.override_defaults(opts.rbls, opts.rules)
 end
 
-if(opts['local_exclude_ip_map'] ~= nil) then
+if (opts['local_exclude_ip_map'] ~= nil) then
   local_exclusions = lua_maps.map_add(N, 'local_exclude_ip_map', 'radix',
-    'RBL exclusions map')
+      'RBL exclusions map')
 end
 
-for key,rbl in pairs(opts.rbls ) do
+-- TODO: this code should be universal for all modules that use selectors to allow
+-- maps usage from selectors registered for a specific module
+if type(opts.attached_maps) == 'table' then
+  opts.attached_maps_processed = {}
+  for i, map in ipairs(opts.attached_maps) do
+    -- Store maps in the configuration table to keep lifetime track
+    opts.attached_maps_processed[i] = lua_maps.map_add_from_ucl(map)
+    if opts.attached_maps_processed[i] == nil then
+      rspamd_logger.warnx(rspamd_config, "cannot parse attached map: %s", map)
+    end
+  end
+end
+
+for key, rbl in pairs(opts.rbls) do
   if type(rbl) ~= 'table' or rbl.disabled == true or rbl.enabled == false then
     rspamd_logger.infox(rspamd_config, 'disable rbl "%s"', key)
   else
@@ -1213,7 +1327,7 @@ for key,rbl in pairs(opts.rbls ) do
     end
     -- Propagate default options from opts to rule
     if not rbl.ignore_defaults then
-      for default_opt_key,_ in pairs(rbl_common.default_options) do
+      for default_opt_key, _ in pairs(rbl_common.default_options) do
         local rbl_opt = default_opt_key:sub(#('default_') + 1)
         if rbl[rbl_opt] == nil then
           rbl[rbl_opt] = opts[default_opt_key]
@@ -1225,15 +1339,19 @@ for key,rbl in pairs(opts.rbls ) do
       rbl.requests_limit = rspamd_config:get_dns_max_requests()
     end
 
-    local res,err = rbl_common.rule_schema:transform(rbl)
+    local res, err = rbl_common.rule_schema:transform(rbl)
     if not res then
       rspamd_logger.errx(rspamd_config, 'invalid config for %s: %s, RBL is DISABLED',
           key, err)
     else
       res = rbl_common.convert_checks(res)
       -- Aliases
-      if res.return_codes then res.returncodes = res.return_codes end
-      if res.return_bits then res.returnbits = res.return_bits end
+      if res.return_codes then
+        res.returncodes = res.return_codes
+      end
+      if res.return_bits then
+        res.returnbits = res.return_bits
+      end
 
       if not res then
         rspamd_logger.errx(rspamd_config, 'invalid config for %s: %s, RBL is DISABLED',
@@ -1254,10 +1372,12 @@ local function rbl_callback_white(task)
     local ws = task:get_symbol(w)
     if ws and ws[1] then
       ws = ws[1]
-      if not ws.options then ws.options = {} end
-      for _,opt in ipairs(ws.options) do
-        local elt,what = opt:match('^([^:]+):([^:]+)')
-        lua_util.debugm(N, task,'found whitelist from %s: %s(%s)', w,
+      if not ws.options then
+        ws.options = {}
+      end
+      for _, opt in ipairs(ws.options) do
+        local elt, what = opt:match('^([^:]+):([^:]+)')
+        lua_util.debugm(N, task, 'found whitelist from %s: %s(%s)', w,
             elt, what)
         if elt and what then
           whitelisted_elements[elt] = {
@@ -1279,20 +1399,22 @@ local function rbl_callback_fin(task)
   lua_util.debugm(N, task, "finished rbl processing")
 end
 
-rspamd_config:register_symbol{
+rspamd_config:register_symbol {
   type = 'callback',
   callback = rbl_callback_white,
   name = 'RBL_CALLBACK_WHITE',
   flags = 'nice,empty,no_squeeze',
-  augmentations = {string.format("timeout=%f", rspamd_config:get_dns_timeout() or 0.0)},
+  groups = { 'rbl' },
+  augmentations = { string.format("timeout=%f", rspamd_config:get_dns_timeout() or 0.0) },
 }
 
-rspamd_config:register_symbol{
+rspamd_config:register_symbol {
   type = 'callback',
   callback = rbl_callback_fin,
   name = 'RBL_CALLBACK',
   flags = 'empty,no_squeeze',
-  augmentations = {string.format("timeout=%f", rspamd_config:get_dns_timeout() or 0.0)},
+  groups = { 'rbl' },
+  augmentations = { string.format("timeout=%f", rspamd_config:get_dns_timeout() or 0.0) },
 }
 
 for _, w in ipairs(white_symbols) do
